UNIT MSClaves;

{
********************************************************************
* Unit MSProdKey                                                   *
*                                                                  *
*  Description: Decode and View the Product Key used to install:   *
*               Windows 2000, XP, Server 2003, Office XP, 2003.    *
*                                                                  *
*                                                                  *                                                         *
* Copyright  2004 Chuck D                                         *
********************************************************************
}

INTERFACE

USES Registry, Windows, SysUtils, Classes;

FUNCTION View_Win_Key: STRING;
FUNCTION View_OXP_Key: STRING;
FUNCTION View_O2K3_Key: STRING;
FUNCTION DecodeProductKey(CONST HexSrc: ARRAY OF BYTE): STRING;

VAR
  Reg: TRegistry;
  binarySize: INTEGER;
  HexBuf: ARRAY OF BYTE;
  temp, temppath, tempkey: TStringlist;
  KeyName, SubKeyName: STRING;

IMPLEMENTATION

FUNCTION View_Win_Key: STRING;
BEGIN
  Reg := TRegistry.Create;
  TRY
       Reg.RootKey := HKEY_LOCAL_MACHINE;
       IF Reg.OpenKey('\SOFTWARE\Microsoft\Windows NT\CurrentVersion', FALSE) THEN
        BEGIN
            IF Reg.GetDataType('DigitalProductId') = rdBinary THEN
             BEGIN
                 binarySize := Reg.GetDataSize('DigitalProductId');
                 SetLength(HexBuf, binarySize);
                 IF binarySize > 0 THEN
                  BEGIN
                      Reg.ReadBinaryData('DigitalProductId', HexBuf[0], binarySize);
                  END;
             END;
        END;
  FINALLY
       FreeAndNil(Reg);
   END;

  Result := '';
  Result := DecodeProductKey(HexBuf);
END;

FUNCTION View_OXP_Key: STRING;
BEGIN
  TRY
       Reg := TRegistry.Create;
       temp := TStringList.Create;
       temppath := TStringList.Create;
       tempkey := TStringList.Create;
       Reg := TRegistry.Create;
       Reg.RootKey := HKEY_LOCAL_MACHINE;
       KeyName := 'SOFTWARE\MICROSOFT\Office\10.0\Registration\';
       Reg.OpenKey(KeyName, FALSE);
       Reg.GetKeyNames(temp);
       Reg.CloseKey;
       SubKeyName := temp.Strings[0];
       Reg.CloseKey;
  EXCEPT on E: EStringListError DO
            Exit
   END;
  TRY
       IF Reg.OpenKey(KeyName + SubKeyName, FALSE) THEN
        BEGIN
            IF Reg.GetDataType('DigitalProductId') = rdBinary THEN
             BEGIN
                 binarySize := Reg.GetDataSize('DigitalProductId');
                 IF binarySize > 0 THEN
                  BEGIN
                      SetLength(HexBuf, binarySize);
                      Reg.ReadBinaryData('DigitalProductId', HexBuf[0], binarySize);
                  END;
             END;
        END;
  FINALLY
       FreeAndNil(Reg);
   END;

  Result := '';
  Result := DecodeProductKey(HexBuf);
END;

FUNCTION View_O2K3_Key: STRING;
BEGIN
  TRY
       Reg := TRegistry.Create;
       temp := TStringList.Create;
       temppath := TStringList.Create;
       tempkey := TStringList.Create;
       Reg := TRegistry.Create;
       Reg.RootKey := HKEY_LOCAL_MACHINE;
       KeyName := 'SOFTWARE\MICROSOFT\Office\11.0\Registration\';
       Reg.OpenKey(KeyName, FALSE);
       Reg.GetKeyNames(temp);
       Reg.CloseKey;
       SubKeyName := temp.Strings[0];
       Reg.CloseKey;
  EXCEPT on E: EStringListError DO
            Exit
   END;
  TRY
       IF Reg.OpenKey(KeyName + SubKeyName, FALSE) THEN
        BEGIN
            IF Reg.GetDataType('DigitalProductId') = rdBinary THEN
             BEGIN
                 binarySize := Reg.GetDataSize('DigitalProductId');
                 IF binarySize > 0 THEN
                  BEGIN
                      SetLength(HexBuf, binarySize);
                      Reg.ReadBinaryData('DigitalProductId', HexBuf[0], binarySize);
                  END;
             END;
        END;
  FINALLY
       FreeAndNil(Reg);
   END;

  Result := '';
  Result := DecodeProductKey(HexBuf);
END;

FUNCTION DecodeProductKey(CONST HexSrc: ARRAY OF BYTE): STRING;
CONST
  StartOffset: INTEGER = $34; { //Offset 34 = Array[52] }
  EndOffset: INTEGER = $34 + 15; { //Offset 34 + 15(Bytes) = Array[64] }
  Digits: ARRAY[0..23] OF CHAR = ('B', 'C', 'D', 'F', 'G', 'H', 'J', 'K', 'M', 'P', 'Q', 'R', 'T', 'V', 'W', 'X', 'Y', '2', '3', '4', '6', '7', '8', '9');
  dLen: INTEGER = 29; { //Length of Decoded Product Key }
  sLen: INTEGER = 15; { //Length of Encoded Product Key in Bytes (An total of 30 in chars) }
VAR
  HexDigitalPID: ARRAY OF CARDINAL;
  Des: ARRAY OF CHAR;
  I, N: INTEGER;
  HN, Value: CARDINAL;
BEGIN
  SetLength(HexDigitalPID, dLen);
  FOR I := StartOffset TO EndOffset DO
   BEGIN
       HexDigitalPID[I - StartOffSet] := HexSrc[I];
   END;

  SetLength(Des, dLen + 1);

  FOR I := dLen - 1 DOWNTO 0 DO
   BEGIN
       IF (((I + 1) MOD 6) = 0) THEN
        BEGIN
            Des[I] := '-';
        END
       ELSE
        BEGIN
            HN := 0;
            FOR N := sLen - 1 DOWNTO 0 DO
             BEGIN
                 Value := (HN SHL 8) OR HexDigitalPID[N];
                 HexDigitalPID[N] := value DIV 24;
                 HN := Value MOD 24;
             END;
            Des[I] := Digits[HN];
        END;
   END;
  Des[dLen] := Chr(0);

  FOR I := 0 TO Length(Des) DO
   BEGIN
       Result := Result + Des[I];
   END;
END;

END.
